var R = R || {};

//
R.convertColor = function(color)
{   
    let r = Phaser.Color.getRed(color);
    let g = Phaser.Color.getGreen(color);
    let b = Phaser.Color.getBlue(color);
    return ((b << 16) | (g << 8) | r) >>> 0;
};

//
R.Palette = class
{
    constructor(colors, cbOnSelect, cbContext)
    {
        this.back = R.createImage(0, game.height, 'palette_back', null, 0, 1);
        let size = this.back.width;
        this.back.scale.set(game.width / size, 128 / size);

        this.colSize = 76;
        this.scaleSelected = 1.15;
        this.scaleUpdating = false;
        this.ds = 0.0125;

        let n = colors.length;
        let x = 10;
        let dx = this.colSize + 24;
        let hs = this.colSize * 0.5;

        this.list = game.add.group();
        this.list.position.set(this.back.x, this.back.y - this.back.height);

        this.stroke = R.createImage(x + hs, 15 + this.colSize * 0.5, 'atlas', 'color');
        this.stroke.idx = -1;
        this.stroke.scale.set(0.9);
        this.stroke.tint = 0xb8bab9;
        this.list.add(this.stroke);

        this.stroke2 = R.createImage(x + hs, 15 + this.colSize * 0.5, 'atlas', 'color');
        this.stroke2.idx = -2;
        this.stroke2.scale.set(0.9);
        this.stroke2.tint = 0xb8bab9;
        this.list.add(this.stroke2);

        for(let i = 0; i < n; ++i)
        {
            let color = R.createImage(x, 15, 'atlas', 'color', 0, 0);
            let number = R.createText(hs, hs + 3, 32, i + 1, '#000000', false, 0, false);
            number.anchor.y = 0.5;
            color.idx = i + 1;
            color.labelIdx = i;
            color.completed = false;
            color.color = colors[i];
            color.tint = R.convertColor(colors[i]);        
            color.addChild(number);
            x += dx;
            this.list.add(color);
        }

        this.scrollView = new R.ScrollView(this.list, true, false, this.onItemTap, this, 20, 0);

        this.selectedItem = null;

        this.cbOnSelect = cbOnSelect;
        this.cbContext = cbContext;

        this.tween = null;
        this.allowSelect = true;
        this.countCompleted = 0;
    }

    //
    onItemTap(item)
    {
        if(!item.completed && item.idx >= 0 && this.selectedItem != item)
        {
            if(this.selectedItem != null)
            {
                this.stroke2.visible = true;
                this.stroke2.x = this.stroke.x;
                this.stroke2.scale.set(this.stroke.scale.x);
            }

            this.stroke.visible = true;
            this.stroke.scale.set(1);
            this.stroke.x = item.x + this.colSize * 0.5;
            this.selectedItem = item;
            this.scaleUpdating = true;

            if(this.cbOnSelect != null && this.cbContext != null) this.cbOnSelect.call(this.cbContext, item);
        }
    }

    //
    update()
    {
        if(this.scaleUpdating) this.updateScale();
        this.scrollView.update();
    }

    //
    isTapOver(y)
    {
        return y > game.height - this.back.height;
    }

    //
    inputOnDown(pointer)
    {
        if(this.tween != null && this.tween.isRunning) this.tween.stop();
        this.scrollView.inputOnDown(pointer);
    }

    //
    inputOnUp(pointer)
    {
        this.scrollView.inputOnUp(pointer);
    }

    //
    updateScale()
    {
        let s1 = this.stroke.scale.x;
        s1 += this.ds;

        let s2 = this.stroke2.scale.x;
        s2 -= this.ds;

        if(s1 >= this.scaleSelected) s1 = this.scaleSelected;
        if(s2 <= 1) s2 = 1;

        this.stroke.scale.set(s1);
        this.stroke2.scale.set(s2)

        if(s2 == 1 && s1 == this.scaleSelected)
        {
            this.stroke2.visible = false;
            this.scaleUpdating = false;
            this.checkOffset(this.selectedItem);
        }
    }

    //
    checkOffset(item)
    {
        if(item.idx < this.list.children.length - 1)
        {
            let i = item.idx == this.list.children.length - 2 ? 1 : 2;
            let x = this.list.children[item.idx + i].x + this.colSize + 16 + this.list.x;
            if(x > game.width) this.tween = game.add.tween(this.list).to({ x: this.list.x - (x - game.width) }, 200, Phaser.Easing.Linear.None, true);
        }

        if(item.idx > 0)
        {
            let x = (item.idx == 1 && this.list.x < 0) ? this.list.x : this.list.children[item.idx].x - 10 + this.list.x
            if(x < 0) this.tween = game.add.tween(this.list).to({ x: this.list.x - x }, 200, Phaser.Easing.Linear.None, true);
        }
    }

    //
    markSelectedAsCompleted(withTween)
    {
        let number = this.selectedItem.children[0];
        if(withTween) game.add.tween(number).to({ alpha: 0 }, 250, Phaser.Easing.Linear.None, true).onComplete.add(number.destroy, number);
        else number.alpha = 0;

        ++this.countCompleted;

        if(this.countCompleted < this.list.length - 2 && this.selectedItem.idx < this.list.children.length - 2)
        {
            if(withTween) this.allowSelect = false;

            let item = this.selectedItem;
            let i = this.list.children.length;

            let x1 = this.list.children[this.list.children.length - 1].x;

            while(i-- > item.idx + 2)
            {
                if(withTween) game.add.tween(this.list.children[i]).to({ x: this.list.children[i - 1].x }, 250, Phaser.Easing.Linear.None, true);
                else this.list.children[i].x = this.list.children[i - 1].x;
                --this.list.children[i].idx;
            }                        

            if(withTween)
            {
                game.add.tween(item).to({ y: item.y - this.colSize - 10 }, 250, Phaser.Easing.Linear.None, true);
                game.add.tween(item.scale).to({ x: 0.6, y: 0.6 }, 250, Phaser.Easing.Linear.None, true);
                
                let t = Math.min(Math.abs(x1 - item.x) / 3, 500);

                game.add.tween(item).to({ x: x1 }, t, Phaser.Easing.Linear.None, true, 250).onComplete.add(function()
                {
                    game.add.tween(item.scale).to({ x: 1, y: 1 }, 250, Phaser.Easing.Linear.None, true);
                    game.add.tween(item).to({ y: item.y + this.colSize + 10 }, 250, Phaser.Easing.Linear.None, true).onComplete.add(function() { this.allowSelect = true; }, this);
                }, this);
            }
            else item.x = x1;

            this.list.bringToTop(item);
            item.idx = this.list.length;
        }        

        this.selectedItem.frameName = 'completed_color';
        this.selectedItem.completed = true;
        this.selectedItem = null;
        this.stroke.visible = false;
    }

    //
    allColorsAreCompleted()
    {
        return this.countCompleted == this.list.length - 2;
    }

    //
    hide(duration)
    {
        game.add.tween(this.back).to({ y: this.back.y + this.back.height }, duration, Phaser.Easing.Linear.None, true);
        game.add.tween(this.list).to({ y: this.list.y + this.back.height }, duration, Phaser.Easing.Linear.None, true);
    }
}