var R = R || {};

//
var PlayState = {

    bmd: null,
    bmdContainer: null,
    currentScale: 0.1,
    wasSingleTouch: false,
    wasPaletteTouch: false,

    zoomStep: 0.05,
    zoomMin: 0.472,
    zoomMax: 3.5,

    ox: 0,
    oy: 0,

    picOX: 0,
    picOY: 0,

    dragging: false,
    pinching: false,
    touches: 0,
    prevFingerDistance: 0,

    pixels: null,
    palette: null,
    imageData: null,
    hittedPixels: null,

    btnBack: null,
    btnBulb: null,
    btnAdjustPic: null,

    btnSkip: null,
    btnSave: null,
    btnWatch: null,
    btnContinue: null,

    labelSkip: null,
    labelSave: null,
    labelWatch: null,
    labelContinue: null,
    message: null,

    tweenMove: null,
    tweenScale: null,
    hintsBar: null,
    btnFlyHint: null,
    timerFlyHint: null,
    stepColorWatching: 0,

    checkStartedPic: true,
    watching: false,

    numbers: null,
    showAd: false,

    //
    shutdown: function()
    {
        this.currentScale = 0.1;
        this.dragging = false;
        this.pinching = false,
        this.touches = 0;
        this.prevFingerDistance = 0;
        this.wasSingleTouch = false;
        this.wasPaletteTouch = false;

        this.bmd.destroy();
        this.bmd = null;       
        this.bmdContainer = null;
        this.pixels = null;
        this.imageData = null;
        this.hittedPixels = null;

        this.btnBack = null;
        this.btnBulb = null;
        this.btnAdjustPic = null;
        this.tweenMove = null;
        this.tweenScale = null;
        this.hintsBar = null;

        this.btnSkip = null;
        this.btnSave = null;
        this.btnWatch = null;

        this.labelSkip = null;
        this.labelSave = null;
        this.labelWatch = null;
        this.labelContinue = null;
        this.btnFlyHint = null;
        this.timerFlyHint = null;
        this.message = null;
        this.numbers = null;
        this.showAd = false;

        if(game.device.desktop) game.input.mouse.mouseWheelCallback = null;

        R.selectedItem = null;
    },

    //
    create: function()
    {       
        this.currentScale = this.zoomMin;

        //
        this.bmd = this.add.bitmapData();
        this.bmd.load(R.pictureKey);

        let w = this.bmd.width;
        let h = this.bmd.height;
        let i = w * h;
        let colorLine = 0xff000000;
        
        this.pixels = new Array(i);

        let colors = [];

        //draw lines
        while(i--)
        {            
            this.pixels[i] = this.bmd.pixels[i];
            if(this.pixels[i] == 0) this.bmd.pixels[i] = colorLine;
            else
            {
                this.addUniqColor(this.bmd.pixels[i], colors);
                this.bmd.pixels[i] = 0;                
            }
        }

        //outlines 2 pixels
        i = w;
        while(i--)
        {
            let i2 = i + (w - 1) * h;
            this.pixels[i] = 0; this.bmd.pixels[i] = colorLine;
            this.pixels[i + w] = 0; this.bmd.pixels[i + w] = colorLine;
            this.pixels[i2] = 0; this.bmd.pixels[i2] = colorLine;
            this.pixels[i2 - w] = 0; this.bmd.pixels[i2 - w] = colorLine;

            i2 = i * w;
            this.pixels[i2] = 0; this.bmd.pixels[i2] = colorLine;            
            this.pixels[i2 + 1] = 0; this.bmd.pixels[i2 + 1] = colorLine;
            i2 += w - 1;
            this.pixels[i2] = 0; this.bmd.pixels[i2] = colorLine; 
            this.pixels[i2 - 1] = 0; this.bmd.pixels[i2 - 1] = colorLine; 
        }

        this.bmd.redraw();
        this.imageData = new Uint8Array(this.bmd.imageData.data);       

        this.bmdContainer = this.bmd.addToWorld();
        this.bmdContainer.scale.set(this.currentScale);
        this.bmdContainer.anchor.set(0.5);
        this.bmdContainer.x = game.width * 0.5;
        this.bmdContainer.y = game.height * 0.5;
        //
        this.palette = new R.Palette(colors, this.onColorSelect, this);

        //numbers
        let numbersJSON = game.cache.getJSON(R.pictureKey);
        let length = numbersJSON.length;
        this.numbers = new Array(length);
        for(i = 0; i < length; ++i)
        {
            let nbrs = numbersJSON[i];
            let l = nbrs.length;
            let labels = [];            
            while(l--)
            {               
                let label = R.createText(nbrs[l].x, nbrs[l].y, 50, (i + 1), '#000000');  
                label.activeSelf = true;
                label.scale.set(nbrs[l].s);                
                this.bmdContainer.addChild(label);
                labels.push(label);
            }
            this.numbers[i] = labels;
        }

        //
        this.hittedPixels = R.loadPicProgress(R.pictureKey);
        if(this.hittedPixels.length > 0) this.loadProgress();
        else this.checkStartedPic = true;

        this.checkNumbersVisibility();

        //        
        this.btnBack = R.createButton(10, 10, 'atlas', 'btn_arrow', 'btn_arrow_pushed', this.onBackButton, this, 0, 0);        
        this.btnBulb = R.createButton(game.width - 10, 10, 'atlas', 'btn_bulb', 'btn_bulb_pushed', this.onBulbButton, this, 1, 0);       
        this.hintsBar = R.createText(-75, 66, 28, R.playerData.hints, '#923d66');
        this.btnBulb.addChild(this.hintsBar);        
        this.btnAdjustPic = R.createButton(game.width - 90, game.height - 220, 'atlas', 'btn_default_size', 'btn_default_size_pushed', this.onAdjustPicButton, this, 0, 0);   
        this.btnAdjustPic.alpha = 0;
        this.btnAdjustPic.inputEnabled = false;

        this.watching = false;

        //message
        let message = R.createImage(game.width * 0.5, 120, 'atlas', 'substrate_0');
        message.tint = 0xd8d7dd;
        message.scale.set(0.8, 1.4 * 0.8);
        message.addChild(R.createText(0, 2, 20, R.strings['select_color'], '#2d2d2d', message.width * message.scale.x - 10));
        message.children[0].scale.set(1 / message.scale.x, 1 / message.scale.y);
        message.visible = false;
        this.message = message;

        //
        game.input.maxPointers = 2;
        game.input.onDown.add(this.inputOnDown, this);
        game.input.onUp.add(this.inputOnUp, this);

        if(game.device.desktop) game.input.mouse.mouseWheelCallback = this.mouseWheelCallback;

        //
        if(R.getPicStatus(R.pictureIdx) == 2) this.completeStage();
        else this.resetFlyTimer();       

        //
        R.showingAd = false;
        Publisher.showAd();
    },

    //
    mouseWheelCallback: function(wheel)
    {
        PlayState.checkAdjustTweens();

        let s1 = PlayState.currentScale;
        let dz = wheel.deltaY * 0.01 * PlayState.zoomStep;
        PlayState.currentScale -= dz;
        if(PlayState.currentScale < PlayState.zoomMin) PlayState.currentScale = PlayState.zoomMin;
        else if(PlayState.currentScale > PlayState.zoomMax) PlayState.currentScale = PlayState.zoomMax;

        let x = game.input.activePointer.x - PlayState.bmdContainer.x;
        let y = game.input.activePointer.y - PlayState.bmdContainer.y;       
        let ts = PlayState.bmd.width;

        PlayState.bmdContainer.x += (x + ts * PlayState.currentScale) - (x + ts * s1) / s1 * PlayState.currentScale;  
        PlayState.bmdContainer.y += (y + ts * PlayState.currentScale) - (y + ts * s1) / s1 * PlayState.currentScale;         

        PlayState.bmdContainer.scale.set(PlayState.currentScale);

        PlayState.clampMoving();

        PlayState.checkVisibilityAdjustButton();

        PlayState.checkNumbersVisibility();
    },

    //
    addUniqColor: function(color, allColors)
    {
        let i = allColors.length;
        while(i--) if(allColors[i] == color) return;
        allColors.push(color);        
    },

    //
    resize: function(width, height)
    {
        
    },

    //
    inputOnDown: function(pointer, event)
    {
        if(game.paused) return;

        if(this.palette.isTapOver(pointer.y))
        {
            this.wasPaletteTouch = true;
            this.palette.inputOnDown(pointer);
            return;
        }

        if(this.buttonsContains(pointer.x, pointer.y)) return;

        this.checkAdjustTweens();

        if(!game.device.desktop) ++this.touches;

        if(game.device.desktop || this.touches == 1)
        {
            this.ox = pointer.x;
            this.oy = pointer.y;
            this.wasSingleTouch = true;
        }
        else if(!this.wasPaletteTouch && this.touches == 2)
        {
            this.wasSingleTouch = false;
            this.dragging = false; 
            this.pinching = true;
            this.prevFingerDistance = Phaser.Point.distance(game.input.pointer1, game.input.pointer2);
        }
    },

    //
    inputOnUp: function(pointer, event)
    {
        if(game.paused) return;

        if(this.wasPaletteTouch)
        {
            this.palette.inputOnUp(pointer);
            this.wasPaletteTouch = false;
            return;
        }
       
        if(this.wasSingleTouch)
        {
            this.tryFill(pointer);
            this.wasSingleTouch = false;            
        }

        this.dragging = false;
        this.pinching = false;

        if(!game.device.desktop && --this.touches < 0) this.touches = 0;        
    },

    update: function()
    {
        if(this.wasSingleTouch) this.detectDrag();        

        if(this.dragging) this.draggingUpdate();

        if(this.pinching) this.pinchingUpdate();

        // if need update
        this.palette.update();
    },

    //
    detectDrag: function()
    {
        let pointer = game.input.activePointer;
        if(Math.abs(this.ox - pointer.x) > R.dragThreshold || Math.abs(this.oy - pointer.y) > R.dragThreshold)
        {
            this.wasSingleTouch = false;
            this.ox = pointer.x;
            this.oy = pointer.y;
            this.picOX = this.bmdContainer.x;
            this.picOY = this.bmdContainer.y;
            this.dragging = true;
        }
    },

    //
    draggingUpdate: function()
    {
        let pointer = game.input.activePointer;
        this.bmdContainer.x = this.picOX + pointer.x - this.ox;
        this.bmdContainer.y = this.picOY + pointer.y - this.oy;
        this.clampMoving();
    },

    clampMoving: function()
    {
        let cx = game.width * 0.5;
        let cy = game.height * 0.5;

        let w = this.bmdContainer.width * 0.5;
        let h = this.bmdContainer.height * 0.5;

        if(this.bmdContainer.x < cx - w) this.bmdContainer.x = cx - w;
        else if(this.bmdContainer.x > cx + w) this.bmdContainer.x = cx + w;

        if(this.bmdContainer.y < cy - h) this.bmdContainer.y = cy - h;
        else if(this.bmdContainer.y > cy + h) this.bmdContainer.y = cy + h;
    },

    pinchingUpdate: function()
    {
        let distance = Phaser.Point.distance(game.input.pointer1, game.input.pointer2);
        let dz = (distance - this.prevFingerDistance) * 0.0025;        

        if(dz !== 0)
        {
            let prevScale = this.currentScale;
            this.currentScale += dz;
            if(this.currentScale < this.zoomMin) this.currentScale = this.zoomMin;
            else if(this.currentScale > this.zoomMax) this.currentScale = this.zoomMax;

            let px = (game.input.pointer2.x - game.input.pointer1.x) * 0.5;
            let py = (game.input.pointer2.y - game.input.pointer1.y) * 0.5;

            let x = game.input.pointer1.x + px - this.bmdContainer.x;
            let y = game.input.pointer1.y + py - this.bmdContainer.y;
            let size1 = this.bmdContainer.width;
            let size2 = this.bmd.width * this.currentScale;

            this.bmdContainer.x += (x + size2) - (x + size1) / prevScale * this.currentScale;
            this.bmdContainer.y += (y + size2) - (y + size1) / prevScale * this.currentScale;

            this.bmdContainer.scale.set(this.currentScale);

            this.clampMoving();

            this.checkVisibilityAdjustButton();
            this.checkNumbersVisibility();
        }

        this.prevFingerDistance = distance;
    },

    //
    tryFill: function(pointer)
    {
        if(this.palette.selectedItem == null)
        {
            this.showMessage();
            return;
        }

        if(!this.palette.allowSelect) return;

        let rs = 1 / this.currentScale;

        let x = Math.floor((pointer.x - this.bmdContainer.x + this.bmdContainer.width * 0.5) * rs);
        let y = Math.floor((pointer.y - this.bmdContainer.y + this.bmdContainer.height * 0.5) * rs);

        if(x < 0 || x >= this.bmd.width || y < 0 || y >= this.bmd.height) return;

        let color = this.palette.selectedItem.color;        
        let i = x + y * this.bmd.width;

        if(color == this.pixels[i]) this.fill(x, y, color);
        else if(this.bmd.pixels[i] == 0xff000000) this.checkFillInRadius(x, y, color, 3);        
    },

    fill: function(x, y, color)
    {
        this.bmd.floodFill(x, y, color, this.imageData, this.pixels, true);
        this.hittedPixels.push({ idx: x + y * this.bmd.width, color: color });
        this.checkNumbers(this.palette.selectedItem.labelIdx);
        this.checkRemainedSelectedColor(this.palette.selectedItem.color, true);        

        if(this.checkStartedPic)
        {
            R.setPicStatus(R.pictureIdx, 1);
            R.savePlayeData();
            this.checkStartedPic = false;
        }

        if(this.timerFlyHint != null) this.resetFlyTimer();
    },

    checkFillInRadius: function(x, y, color, r)
    {        
        let xt = x + r + 1;
        let yt = y + r + 1;
        let size = this.bmd.width;

        for(let y0 = y - r; y0 < yt; ++y0)
        {
            if(y0 < 0) continue;
            if(y0 >= size) break;
            for(let x0 = x - r; x0 < xt; ++x0)
            {
                if(x0 < 0) continue;
                if(x0 >= size) break;                
                if(this.pixels[x0 + y0 * size] == color)
                {
                    this.fill(x0, y0, color);
                    return;
                }
            }
        }        
    },

    checkNumbers: function(idx)
    {
        let width = this.bmd.width;
        let hs = width * 0.5;
        let labels = this.numbers[idx];      
        let i = labels.length;
        while(i--)
        {
            if(labels[i].activeSelf)
            {
                if(this.pixels[labels[i].x + hs + (labels[i].y + hs) * width] == 0)
                {
                    labels[i].activeSelf = false;
                    labels[i].visible = false;
                }
            }
        }
    },

    checkNumbersVisibility: function()
    {
        let i = this.bmdContainer.children.length;      
        let s = this.bmdContainer.scale.x;
        let sz = s > this.zoomMin * 1.15;        
        while(i--)
        {
            let label = this.bmdContainer.children[i];
            if(label.activeSelf) label.visible = sz && (label.height * s > 20);
        }
        
    },

    deselectAreas: function()
    {
        if(this.watching) this.onSkipButton();       

        let i = this.pixels.length;
        while(i--) if(this.pixels[i] != 0) this.bmd.pixels[i] = 0;
        this.bmd.redraw();
    },

    onColorSelect: function(item)
    {       
        let color = item.color;
        let w = this.bmd.width;
        let h = this.bmd.height;
        let i = 0;

        let ca = 0xffdcdcdc;
        let cb = 0xffaaaaaa;
        let fillColor = cb;        
        let qSize = 12;
        let qx = 0;
        let qy = 0;

        for(let y = 0; y < h; ++y)
        {
            for(let x = 0; x < w; ++x)
            {
                i = x + y * w;

                if(this.pixels[i] == color) this.bmd.pixels[i] = fillColor;
                else if(this.pixels[i] != 0) this.bmd.pixels[i] = 0;

                if(++qx == qSize)
                {
                    qx = 0;
                    fillColor = fillColor == cb ? ca : cb;
                }
            }

            qx = 0;                     

            if(++qy == qSize)
            {
                qy = 0;                
                fillColor = fillColor == cb ? ca : cb;
            }         
        }

        this.bmd.redraw();
    },

    checkRemainedSelectedColor: function(color)
    {        
        let i = this.bmd.width * this.bmd.height;
        while(i--) if(this.pixels[i] == color) return;
        this.palette.markSelectedAsCompleted(true);

        if(this.palette.allColorsAreCompleted()) this.completeStage();
    },

    buttonsContains: function(x, y)
    {
        if(this.btnFlyHint != null && this.btnFlyHint.button.getBounds().contains(x, y)) return true;
        if(this.btnAdjustPic.alpha > 0 && this.btnAdjustPic.getBounds().contains(x, y)) return true;
        return this.btnBack.getBounds().contains(x, y) || this.btnBulb.getBounds().contains(x, y);
    },

    checkVisibilityAdjustButton: function()
    {
        if(this.currentScale - this.zoomMin > 0.2)
        {
            if(!this.btnAdjustPic.inputEnabled)
            {
                game.add.tween(this.btnAdjustPic).to({ alpha: 1.0 }, 250, Phaser.Easing.Linear.None, true);
                this.btnAdjustPic.inputEnabled = true;
            }
        }
        else
        {
            if(this.btnAdjustPic.inputEnabled)
            {
                game.add.tween(this.btnAdjustPic).to({ alpha: 0.0 }, 250, Phaser.Easing.Linear.None, true);
                this.btnAdjustPic.inputEnabled = false;
            }
        }
    },

    checkAdjustTweens: function()
    {
        if(this.tweenMove != null && this.tweenMove.isRunning)
        {
            this.tweenMove.stop();
            this.tweenScale.stop();
            this.currentScale = this.bmdContainer.scale.x;
            this.checkVisibilityAdjustButton();
            this.checkNumbersVisibility();
        }
    },

    completeStage: function()
    {
        if(this.btnSkip == null)
        {
            if(this.timerFlyHint != null)
            {
                game.time.events.remove(this.timerFlyHint);
                this.timerFlyHint = null;
            }

            if(this.btnFlyHint != null) 
            {
                this.btnFlyHint.button.destroy(true);
                this.btnFlyHint = null;
            }

            game.input.maxPointers = 1;
            game.input.onDown.remove(this.inputOnDown, this);
            game.input.onUp.remove(this.inputOnUp, this);
            if(game.device.desktop) game.input.mouse.mouseWheelCallback = null;

            this.btnAdjustPic.inputEnabled = this.btnBack.inputEnabled = this.btnBulb.inputEnabled = false;
            let x = game.width * 0.5;
            this.btnSkip = R.createButton(x, game.height - 200, 'atlas', 'btn_playback', 'btn_playback_push', this.onSkipButton, this);
            this.labelSkip = R.createText(x, this.btnSkip.y + 70, 30, R.strings['skip'], '#d5d5d5');
            this.labelSkip.alpha = this.btnSkip.alpha = 0;
            this.palette.hide(500);
            this.onAdjustPicButton(); 

            game.add.tween(this.btnBack).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
            game.add.tween(this.btnBulb).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
            game.add.tween(this.hintsBar).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);

            R.setPicStatus(R.pictureIdx, 2);
            R.savePlayeData();

            //
            Publisher.analyticsLevelCompleted(R.pictureIdx);
            this.showAd = true;
        }
        else this.btnSkip.inputEnabled = true;
        
        game.add.tween(this.btnSkip).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true, 500);
        game.add.tween(this.labelSkip).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true, 500);              

        //
        let colorLine = 0xff000000;
        let i = this.pixels.length;
        while(i--) if(this.bmd.pixels[i] != colorLine) this.bmd.pixels[i] = 0;
        this.bmd.redraw();

        //
        this.watchProcess(600);
    },

    onBulbButton: function()
    {
        if(this.palette.selectedItem == null)
        {
            this.showMessage();
            return;
        }

        if(R.playerData.hints == 0)
        {
            //show reward
            window.Booster.RewardedVideo.play(this.onAdCompleted, this);
            return;
        }

        let color = this.palette.selectedItem.color;
        let l = this.pixels.length;
        let i = 0;
        while(i < l) 
        {
            if(color == this.pixels[i])
            {
                let y = Math.floor(i / this.bmd.width);
                let x = i - y * this.bmd.width;
                let s = this.zoomMax;

                let tx = game.width * 0.5 + (this.bmd.width * 0.5 - x) * s;
                let ty = game.height * 0.5 + (this.bmd.height * 0.5 - y) * s;                

                this.tweenMove = game.add.tween(this.bmdContainer).to({ x: tx, y: ty }, 1000, Phaser.Easing.Quadratic.Out, true);
                this.tweenScale = game.add.tween(this.bmdContainer.scale).to({ x: s, y: s }, 1000, Phaser.Easing.Quadratic.Out, true);
                this.tweenScale.onUpdateCallback(this.checkNumbersVisibility, this);
                this.currentScale = s;

                this.checkVisibilityAdjustButton();

                break;
            }
            ++i;
        }
        
        this.hintsBar.text = --R.playerData.hints;
        R.savePlayeData();
    },    

    onAdjustPicButton: function()
    {
        let y = this.btnBulb.inputEnabled ? game.height * 0.5 : game.height * 0.35;

        this.tweenMove = game.add.tween(this.bmdContainer).to({ x: game.width * 0.5, y: y }, 500, Phaser.Easing.Linear.None, true);
        this.tweenScale = game.add.tween(this.bmdContainer.scale).to({ x: this.zoomMin, y: this.zoomMin }, 500, Phaser.Easing.Linear.None, true);
        this.tweenScale.onUpdateCallback(this.checkNumbersVisibility, this);
        this.tweenScale.onComplete.add(this.checkNumbersVisibility, this);
        this.currentScale = this.zoomMin;

        game.add.tween(this.btnAdjustPic).to({ alpha: 0.0 }, 250, Phaser.Easing.Linear.None, true);
        this.btnAdjustPic.inputEnabled = false;
    },
    
    onBackButton: function()
    {
        if(this.hittedPixels.length > 0)
        {
            this.deselectAreas();
            R.savePicProgress(this.hittedPixels, R.pictureKey);
            R.savePlayerThumbnail(this.bmd);
        }

        game.state.start('gallery');

        //
        Publisher.showAd();
    },

    //completed stage
    onSkipButton: function()
    {
        game.time.events.removeAll();        
        this.makePicCompleted();
        this.onWachingComplete();
    },

    makePicCompleted: function()
    {
        let i = this.pixels.length;
        let colorLine = 0xff000000;
        while(i--) if(this.bmd.pixels[i] == colorLine) this.pixels[i] = colorLine;
        this.bmd.load(R.pictureKey);
        i = this.pixels.length;
        while(i--) if(this.pixels[i] == colorLine) this.bmd.pixels[i] = colorLine;
        this.bmd.redraw();
    },

    watchProcess: function(delay)
    {
        this.watching = true;
        this.stepColorWatching = 0;       
        game.time.events.add(delay, this.step, this);
    },

    fromHittedFill: function(i)
    {
        let idx = this.hittedPixels[i].idx;
        let y = Math.floor(idx / this.bmd.width);
        let x = idx - y * this.bmd.width;
        this.bmd.floodFill(x, y, this.hittedPixels[i].color, this.imageData, this.pixels, true);
    },

    step: function()
    {
        if(this.stepColorWatching < this.hittedPixels.length)
        {
            this.fromHittedFill(this.stepColorWatching++);            
            game.time.events.add(100, this.step, this);
        }
        else this.onWachingComplete();
    },

    onWachingComplete: function()
    {
        this.btnSkip.inputEnabled = false;
        game.add.tween(this.btnSkip).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true);
        game.add.tween(this.labelSkip).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true); 

        if(this.btnSave == null)
        {
            let c = game.width * 0.5;
            let y = game.height - 240;
            this.btnSave = R.createButton(c - 100, y, 'atlas', 'btn_save', 'btn_save_pushed', this.onSaveButton, this);
            this.btnWatch = R.createButton(c + 100, y, 'atlas', 'btn_watch', 'btn_watch_pushed', this.onWatchButton, this);
            this.btnContinue = R.createButton(c, y + 160, 'atlas', 'substrate_1', 'substrate_1', this.onBackButton, this);

            this.labelSave = R.createText(this.btnSave.x, this.btnSave.y + 70, 30, R.strings['save'], '#f5800c');
            this.labelWatch = R.createText(this.btnWatch.x, this.btnWatch.y + 70, 30, R.strings['watch'], this.labelSave.fill);
            this.labelContinue = R.createText(this.btnContinue.x, this.btnContinue.y, 40, R.strings['continue'], '#ff4774', 390);                       

            this.labelContinue.alpha = this.btnContinue.alpha = this.btnSave.alpha = this.btnWatch.alpha = this.labelSave.alpha = this.labelWatch.alpha = 0;
        }
        else this.btnContinue.inputEnabled = this.btnSave.inputEnabled = this.btnWatch.inputEnabled = true;

        let fun = function() { this.labelContinue.alpha = this.btnContinue.alpha = this.btnWatch.alpha = this.labelSave.alpha = this.labelWatch.alpha = this.btnSave.alpha; };
        game.add.tween(this.btnSave).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true).onUpdateCallback(fun, this).onComplete.add(fun, this); 

        this.watching = false;

        if(this.showAd)
        {
            this.showAd = false;
            Publisher.showAd();
        }
    },

    onSaveButton: function()
    {        
        let link = document.getElementById('linkID');
        link.href = this.bmd.canvas.toDataURL('image/png');
        link.download = 'RelaxColoring_' + R.pictureKey + '.png';
        link.click();
    },

    onWatchButton: function()
    {
        this.btnContinue.inputEnabled = this.btnSave.inputEnabled = this.btnWatch.inputEnabled = false;

        let fun = function() { this.labelContinue.alpha = this.btnContinue.alpha = this.btnWatch.alpha = this.labelSave.alpha = this.labelWatch.alpha = this.btnSave.alpha; };
        game.add.tween(this.btnSave).to({ alpha: 0 }, 200, Phaser.Easing.Linear.None, true).onUpdateCallback(fun, this).onComplete.add(fun, this);

        this.completeStage();
    },

    loadProgress: function()
    {
        let l = this.hittedPixels.length;
        for(let i = 0; i < l; ++i)
        {
            let idx = this.hittedPixels[i].idx;
            let y = Math.floor(idx/ this.bmd.width);
            let x = idx - y * this.bmd.width;

            this.bmd.floodFill(x, y, this.hittedPixels[i].color, this.imageData, this.pixels, false);
        }

        this.bmd.redraw();        


        //check palette colors
        let c = this.palette.list.total;
        while(c-- > 2)        
        {
            let p = this.pixels.length;
            let outContinue = false;
            let color = this.palette.list.children[c].color;
            while(p--)
            {
                if(this.pixels[p] == color)
                {
                    outContinue = true;
                    break;
                }
            }

            //check numbers
            this.checkNumbers(this.palette.list.children[c].labelIdx);

            //
            if(outContinue) continue;

            this.palette.selectedItem = this.palette.list.children[c];
            this.palette.markSelectedAsCompleted(false);

            this.checkStartedPic = false;
        }        
    },

    //fly hints for ad
    resetFlyTimer: function()
    {
        if(this.timerFlyHint != null) game.time.events.remove(this.timerFlyHint);
        this.timerFlyHint = game.time.events.add(10000, this.onTimerFlyHints, this);
    },

    onTimerFlyHints: function()
    {
        this.timerFlyHint = null;
        if(this.btnSkip == null) this.btnFlyHint = new R.BtnFlyHint(50, game.height * 0.3);        
    },

    countdownFlyComplete: function()
    {
        this.btnFlyHint = null;
        if(this.btnSkip == null) this.resetFlyTimer();
    },

    addHints: function()
    {
        R.playerData.hints += 2;
        R.savePlayeData();
        this.hintsBar.text = R.playerData.hints;
    },

    //message
    showMessage: function()
    {
        this.message.children[0].alpha = this.message.alpha = 0;
        this.message.visible = true;

        game.add.tween(this.message).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);
        game.add.tween(this.message.children[0]).to({ alpha: 1 }, 200, Phaser.Easing.Linear.None, true);

        game.time.events.add(3200, this.hideMessage, this);
    },

    hideMessage: function()
    {
        game.add.tween(this.message).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true);
        game.add.tween(this.message.children[0]).to({ alpha: 0 }, 500, Phaser.Easing.Linear.None, true).onComplete.add(function() { this.message.visible = false; }, this);
    },

    //ad
    onAdCompleted: function()
    {
        ++R.playerData.hints;
        this.onBulbButton();
    }
};