var R = R || {};

//
R.createText = function(x, y, size, text, fill, wrap, wordWrapWidth )
{
    if(!fill) fill = '#ffffff';
    let label = game.add.text(x, y, text);
    label.font = R.fontName;
    label.anchor.set(0.5);
    label.align = 'center';
    label.fontSize = size;
    label.fill = fill;

    if(wrap != undefined && wrap > 4 && label.width > wrap) label.scale.set(wrap / label.width);

    if(wordWrapWidth != undefined && wordWrapWidth > 4)
    {
        label.wordWrap = true;
        label.wordWrapWidth = wordWrapWidth;
    }

    return label;
};

//
R.createButton = function(x, y, atlas, frameNormal, framePushed, cb, cbContext, ax, ay)
{
    if(ax === undefined) ax = 0.5;
    if(ay === undefined) ay = ax;
    let button = game.add.button(x, y, atlas, cb, cbContext, frameNormal, frameNormal, framePushed, frameNormal);
    button.anchor.set(ax, ay);
    return button;
};

//GUI
R.ui = {  

    buttonsEnabled: function(group, enabled)
    {
        let i = group.total;
        while(i--) if(group.children[i].type === Phaser.BUTTON) group.children[i].inputEnabled = enabled;
        
    }
};

//BtnFlyHint
R.BtnFlyHint = class
{
    constructor(x, y)
    {
        let button = game.add.sprite(-100, y, 'atlas', 'btn_fly_hint1');
        
        button.animations.add('a', ['btn_fly_hint1', 'btn_fly_hint2', 'btn_fly_hint3', 'btn_fly_hint2'], 8, true).play();

        let stroke = R.createImage(0, 0, 'atlas', 'stroke', 0, 0);
        stroke.scale.set(1.05);
        let mask = game.add.graphics(0, 0);        
        stroke.mask = mask;

        let bulb = game.add.sprite(button.width * 0.5, button.height * 0.5 - 6, 'atlas', 'bulb1');
        bulb.anchor.set(0.5);
        bulb.scale.set(1.1, 0.9);
        bulb.animations.add('a', ['bulb1', 'bulb2', 'bulb3', 'bulb12'], 8, true).play();        

        let subst = R.createImage(70, 80, 'atlas', 'btn_counter');
        subst.scale.set(0.85);
        let ad = R.createText(subst.x, subst.y + 3, 20, 'AD', '#923d66');
        
        bulb.addChild(R.createText(0, -14, 17, '+2', '#923d66'));

        button.addChild(stroke);
        button.addChild(mask);
        button.addChild(subst);
        button.addChild(ad);
        button.addChild(bulb);
        
        button.events.onInputDown.add(this.onInputDown, this);      

        this.button = button;
        this.mask = mask;
        this.bulb = bulb;
        this.angle = 0;

        this.drawMask();

        game.add.tween(this.button).to({ x: x }, 500, Phaser.Easing.Back.Out, true).onComplete.add(this.startCountdown, this);        
    }

    drawMask()
    {
        this.mask.clear();
        this.mask.beginFill(0xffffff);
        this.mask.arc(43, 43, 46, -Math.PI * 0.5, this.angle - Math.PI * 0.5, true, 128);
        this.mask.endFill();
    }

    startCountdown()
    {
        this.angle = 0;
        this.button.inputEnabled = true;
        game.add.tween(this).to({ angle: Math.PI * 2 }, 10000, Phaser.Easing.Linear.None, true).onUpdateCallback(this.drawMask, this).onComplete.add(this.countdownComplete, this);
        this.animBulb();
    }

    countdownComplete()
    {
        this.button.inputEnabled = false;
        this.drawMask();
        game.add.tween(this.button).to({ x: -100 }, 500, Phaser.Easing.Back.In, true).onComplete.add(this.killself, this);
        PlayState.countdownFlyComplete();
    }

    killself()
    {
        this.button.destroy(true);
    }

    onInputDown()
    {        
        game.paused = true;
        R.showingAd = true;
        window.Booster.RewardedVideo.play(this.adComplete, this);        
    }

    animBulb()
    {
        game.add.tween(this.bulb).to({ y: this.bulb.y - 12 }, 600, Phaser.Easing.Back.Out, true, 0, 0, true);
        game.add.tween(this.bulb.scale).to({ x: 1, y: 1 }, 600, Phaser.Easing.Back.Out, true, 0, 0, true).onComplete.add(this.launchAnimBulb, this);
    }

    launchAnimBulb()
    {
        game.time.events.add(500, this.animBulb, this);
    }

    adComplete()
    {
        game.paused = false;
        game.input.maxPointers = 2;        
        this.countdownComplete();
        PlayState.addHints();
        R.showingAd = false;
    }
};