var R = R || {};

R.ScrollView = class
{
    constructor(content, horizontal, vertical, cbItemTap, cbContext, rightMargin, bottomMargin)
    {
        this.content = content;
        this.horizontal = horizontal;
        this.vertical = vertical;

        this.velocityX = 0;
        this.velocityY = 0;

        this.oX = 0;
        this.oY = 0;

        this.minX = game.width - content.width - rightMargin;
        this.maxX = content.x;

        if(this.minX > this.maxX) this.minX = this.maxX;

        this.minY = game.height - content.height - bottomMargin;
        this.maxY = content.y;

        if(this.minY > this.maxY) this.minY = this.maxY;

        this.wasInputDown = false;
        this.dragging = false;

        this.cbItemTap = cbItemTap;
        this.cbContext = cbContext;

        this.tweenHor = null;
        this.tweenVer = null;
    }

    //
    inputOnDown(pointer)
    {
        this.oX = pointer.x;
        this.oY = pointer.y;

        this.velocityX = this.velocityY = 0;

        this.dragging = false;
        this.wasInputDown = true;

        if(this.tweenHor != null && this.tweenHor.isRunning) this.tweenHor.stop();
        if(this.tweenVer != null && this.tweenVer.isRunning) this.tweenVer.stop();
    }

    //
    inputOnUp(pointer)
    {
        if(this.dragging) this.moveInertia();
        else if(this.wasInputDown)
        {
            let item = this.getTouchedItem(pointer.x, pointer.y);
            if(item != null) this.cbItemTap.call(this.cbContext, item);            
        }

        this.dragging = this.wasInputDown = false;
    }

    //
    getTouchedItem(x, y)
    {
        let i = this.content.children.length;

        while(i--)
        {
            let rect = this.content.children[i].getBounds();
            if(rect.contains(x, y)) return this.content.children[i];            
        }

        return null;
    }

    //
    detectDrag()
    {
        let pointer = game.input.activePointer;
        if(Math.abs(this.oX - pointer.x) > R.dragThreshold || Math.abs(this.oY - pointer.y) > R.dragThreshold)
        {
            this.wasInputDown = false;
            this.oX = pointer.x;
            this.oY = pointer.y;
            this.dragging = true;
        }
    }

    //
    scrolling()
    {
        let pointer = game.input.activePointer;

        if(this.horizontal)
        {
            let dx = pointer.x - this.oX;
            this.oX = pointer.x;
            let x = this.content.x + dx;
            if((dx > 0 && x > this.maxX) || (dx < 0 && x < this.minX)) dx *= 0.35;

            x = this.content.x;
            this.content.x = x + dx;

            this.velocityX = (this.content.x - x) * R.scrollVelocityMultiplier;
        }

        if(this.vertical)
        {
            let dy = pointer.y - this.oY;
            this.oY = pointer.y;
            let y = this.content.y + dy;
            if((dy > 0 && y > this.maxY) || (dy < 0 && y < this.minY)) dy *= 0.35;

            y = this.content.y;
            this.content.y = y + dy;

            this.velocityY = (this.content.y - y) * R.scrollVelocityMultiplier;
        }
    }

    //
    moveInertia()
    {
        if(this.horizontal)
        {
            this.content.x += this.velocityX;

            if(this.content.x > this.maxX || this.content.x < this.minX)
            {
                this.tweenHor = game.add.tween(this.content).to({ x: this.content.x < 0 ? this.minX : this.maxX }, 250, Phaser.Easing.Quadratic.Out, true);
                this.velocityX = 0;
            }
            else
            {
                this.velocityX *= R.scrollVelocityDamping;
                if(Math.abs(this.velocityX) < 0.25) this.velocityX = 0;
            }
        }

        if(this.vertical)
        {
            this.content.y += this.velocityY;

            if(this.content.y > this.maxY || this.content.y < this.minY)
            {
                this.tweenVer = game.add.tween(this.content).to({ y: this.content.y < 0 ? this.minY : this.maxY }, 250, Phaser.Easing.Quadratic.Out, true);
                this.velocityY = 0;
            }
            else
            {
                this.velocityY *= R.scrollVelocityDamping;
                if(Math.abs(this.velocityY) < 0.25) this.velocityY = 0;
            }
        }
    }

    //
    update()
    {
        if(this.wasInputDown) this.detectDrag();
        if(this.dragging) this.scrolling();
        else if(this.velocityX != 0 || this.velocityY != 0) this.moveInertia();    
    }
}